local MAJOR = "StarLibWidgetBar-1.0" 
local MINOR = 13

assert(LibStub, MAJOR.." requires LibStub") 
local WidgetBar = LibStub:NewLibrary(MAJOR, MINOR)
if not WidgetBar then return end

local LibProperty = LibStub("StarLibProperty-1.0", true)
assert(LibProperty, MAJOR .. " requires StarLibProperty-1.0")
local LibTimer = LibStub("StarLibTimer-1.0", true)
assert(LibTimer, MAJOR .. " requires StarLibTimer-1.0")
local LibError = LibStub("StarLibError-1.0", true)
assert(LibError, MAJOR .. " requires StarLibError-1.0")
local LibWidget = LibStub("StarLibWidget-1.0", true)
assert(LibWidget, MAJOR .. " requires StarLibWidget-1.0")

local pool = setmetatable({}, {__mode = "k"})

if not WidgetBar.__index then
	WidgetBar.__index = WidgetBar
end

local barUpdate

SCHARS = {
    {31, 31, 31, 31, 31, 31, 31, 31},
    {31, 16, 16, 16, 16, 16, 16, 31},
    {31,  0,  0,  0,  0,  0,  0, 31},
    {31,  1,  1,  1,  1,  1,  1, 31},
    {31, 31, 31, 31,  0,  0,  0,  0},
    { 0,  0,  0,  0, 31, 31, 31, 31}};

WidgetBar.DIR_EAST, WidgetBar.DIR_WEST = 1, 2
WidgetBar.STYLE_NORMAL, WidgetBar.STYLE_HOLLOW = 1, 2

WidgetBar.directionList = {"West", "East"}
WidgetBar.styleList = {"Normal", "Hollow"}
	
WidgetBar.defaults = {
	length = 10, height = 1, direction = WidgetBar.DIR_EAST, update = 1000, style = WidgetBar.STYLE_NORMAL
}

local widgetType = {bar = true}

-- Create a new StarLibWidgetBar object
-- @name WidgetBar.New
-- @usage WidgetBar:New(visitor, name, config, row, col, layer, fontString, env, errorLevel)
-- @param visitor An StarLibCore-1.0 object
-- @param config This widget's settings
-- @param row This widget's row
-- @param col This widget's column
-- @param layer This widget's layer
-- @param config The settings.
-- @param errorLevel The errorLevel for this object
-- @param callback The draw function
-- @return A new StarLibWidgetBar object
function WidgetBar:New(visitor, name, config, row, col, layer, errorLevel, callback, data) 

	assert(name, "Please provide the bar with a name")
	assert(config, "Please provide the bar with a config")
	assert(config.expression, name .. ": Please provide the bar with an expression")
			
	local obj = next(pool)

	if obj then
		pool[obj] = nil
	else
		obj = {}
	end
	
	setmetatable(obj, self)
	
	obj.error = LibError:New(MAJOR, errorLevel)
		
	obj.errorLevel = errorLevel or 3

	obj.callback = callback

	obj.widget = LibWidget:New(obj, visitor, name, config, row, col, layer, widgetType, errorLevel)

    obj.lcd_type = visitor.type

    obj.callback = callback
	obj.data = data

    obj.expression = LibProperty:New(obj, visitor, name .. " expression", config.expression, "", errorLevel)
    obj.expression2 = LibProperty:New(obj, visitor, name .. "expression2", config.expression2, "", errorLevel);
    obj.expr_min = LibProperty:New(obj, visitor, name .. " min", config.min, nil, errorLevel)
    obj.expr_max = LibProperty:New(obj, visitor, name .. " max", config.max, nil, errorLevel)

	obj.color1 = LibProperty:New(obj, visitor, name .. " color", config.color1, "", errorLevel)
	obj.color2 = LibProperty:New(obj, visitor, name .. " color", config.color2, "", errorLevel)
	
	obj.color_valid = {}
	
	--[[obj.color = {}
	obj.color_valid = {}
    obj.color_valid[0] = obj.widget:WidgetColor(section, "barcolor0", obj.color[0]);
    obj.color_valid[1] = obj.widget:WidgetColor(section, "barcolor1", obj.color[1]);

	obj.fg_color = {}
	obj.bg_color = {}
    obj.fg_valid = obj.widget:WidgetColor(section, "foreground", obj.fg_color);
    obj.bg_valid = obj.widget:WidgetColor(section, "background", obj.bg_color);
	]]
	
    obj.length = config.length or WidgetBar.defaults.length --visitor.CFG:FetchRaw(config, "length", WidgetBar.defaults.length)

    obj.height = config.height or WidgetBar.defaults.height --visitor.CFG:FetchRaw(config, "height", WidgetBar.defaults.height)

    obj.direction = config.direction or WidgetBar.defaults.direction --visitor.CFG:FetchRaw(config, "direction", WidgetBar.defaults.direction);

    obj.update = config.update or WidgetBar.defaults.update --visitor.CFG:FetchRaw(config, "update", WidgetBar.defaults.update)

    obj.style = config.style or WidgetBar.defaults.style --visitor.CFG:FetchRaw(config, "style", WidgetBar.defaults.style);
	
	obj.texture = config.texture
	
	obj.val1 = 0;
	obj.val2 = 0;
	obj.min = 0;
	obj.max = 0;

	obj.timer = LibTimer:New("WidgetBar.timer " .. obj.widget.name, obj.update, true, barUpdate, obj)
--    QObject::connect(visitor_->GetWrapper(), SIGNAL(_ResizeLCD(int, int, int, int)),
--        this, SLOT(Resize(int, int, int, int)));
		
	return obj	
end

-- Delete a StarLibWidgetBar object
-- @name StarLibWidgetBar.Del
-- @usage StarLibWidgetBar:Del([marq])
-- @param marq A StarLibWidgetBar object
function WidgetBar:Del(bar)
	if type(bar) ~= "table" then bar = self end
	pool[bar] = true
	bar.timer:Del()
	bar.widget:Del()
	bar.error:Del()
end

function WidgetBar:Resize(rows, cols, old_rows, old_cols)
    local xres = self.visitor.lcd.XRES;
    local yres = self.visitor.lcd.YRES
    local y = rows_ * yres / old_rows;
    local x = cols_ * xres / old_cols;
    local r = row_ * yres / old_rows;
    local c = col_ * xres / old_cols;
    self.rows = round(self.visitor.lcd.LROWS * y / yres);
    self.cols = round(self.visitor.lcd.LCOLS * x / xres);
    self.row = round(self.visitor.lcd.LROWS * r / yres);
    self.col = round(self.visitor.lcd.LCOLS * c / xres);
    self:Update();
end

function WidgetBar:Start()
    if( self.update < 0 ) then
        return;
	end
    self.timer:Start();
    self:Update();
end

function WidgetBar:Stop()
    self.timer:Stop();
end

function WidgetBar:Draw()
	if type(self.callback) == "function" then
		self:callback(self.data)
	end
end

function barUpdate(bar) 
	if bar then
		bar:Update()
	end
end

function WidgetBar:Update() 
    self.expression:Eval();
    local val1, valMax = self.expression:P2N();
	
	self.color1:Eval()
	self.color2:Eval()
	
	if type(val1) ~= "number" then 
		val1 = 1
	end
	
    local val2 = val1;
    if( self.expression2.is_valid ) then
        self.expression2:Eval();
        val2 = self.expression2:P2N();
    end

    local max, min;
    if( self.expr_min.is_valid ) then
        self.expr_min:Eval();
        min = self.expr_min:P2N();
    else
        min = self.min;
        if( val1 < min ) then
            min = val1;
		end
        if( val2 < min ) then
            min = val2;
		end
    end

    if( self.expr_max.is_valid ) then
        self.expr_max:Eval();
        max = self.expr_max:P2N();
    else 
        max = self.max;
        if( val1 > max ) then
            max = val1;
		end
        if( val2 > max ) then
            max = val2;
		end
    end
	
    self.min = min;
    self.max = max;

    if( max > min ) then
        self.val1 = (val1 - min ) / (max - min);
        self.val2 = (val2 - min ) / (max - min);
    else
        self.val1 = 0.0;
        self.val2 = 0.0;
    end

    self:Draw();
end

function WidgetBar:GetOptions(visitor, db)
	local defaults = WidgetBar.defaults
	local options = {
		length = {
			name = "Bar length",
			desc = "Enter the bar's length",
			type = "input",
			pattern = "%d",
			get = function() return tostring(db.length or defaults.length) end,
			set = function(info, v) db.length = tonumber(v) end,
			order = 1
		},
		height = {
			name = "Bar height",
			desc = "Enter the bar's height",
			type = "input",
			pattern = "%d",
			get = function() return tostring(db.height or defaults.height) end,
			set = function(info, v) db.height = tonumber(v) end,
			order = 2
		},
		update = {
			name = "Bar update rate",
			desc = "Enter the bar's refresh rate",
			type = "input",
			pattern = "%d",
			get = function() return tostring(db.update or defaults.update) end,
			set = function(info, v) db.update = tonumber(v) end,
			order = 3
		},
		direction = {
			name = "Bar direction",
			type = "select",
			values = WidgetBar.directionList,
			get = function() return db.direction or defaults.direction end,
			set = function(info, v) db.direction = v end,
			order = 4
		},
		style = {
			name = "Bar Style",
			type = "select",
			values = WidgetBar.styleList,
			get = function() return db.style or defaults.style end,
			set = function(info, v) db.style = v end,
			order = 5
		},
		expression = {
			name = "Bar expression",
			desc = "Enter the bar's first expression",
			type = "input",
			multiline = true,
			width = "full",
			get = function() return db.expression end,
			set = function(info, v) db.expression = v end,
			order = 6
		},
		expression2 = {
			name = "Bar second expression",
			desc = "Enter the bar's second expression",
			type = "input",
			multiline = true,
			width = "full",
			get = function() return db.expression2 end,
			set = function(info, v) db.expression2 = v end,
			order = 7
		},
		min = {
			name = "Bar min expression",
			desc = "Enter the bar's minimum expression",
			type = "input",
			multiline = true,
			width = "full",
			get = function() return db.min end,
			set = function(info, v) db.min = v end,
			order = 8
		
		},
		max = {
			name = "Bar max expression",
			desc = "Enter the bar's maximum expression",
			type = "input",
			multiline = true,
			width = "full",
			get = function() return db.max end,
			set = function(info, v) db.max = v end,
			order = 9
		},
		color1 = {
			name = "First bar color script",
			desc = "Enter the bar's first color script",
			type = "input",
			multiline = true,
			width = "full",
			get = function() return db.color1 end,
			set = function(info, v) db.color1 = v end,
			order = 10
		},
		color2 = {
			name = "Second bar color script",
			desc = "Enter the bar's second color script",
			type = "input",
			multiline = true,
			width = "full",
			get = function() return db.color2 end,
			set = function(info, v) db.color2 = v end,
			order = 10
		}		
	}
	return options
end
